@@ -31,7 +31,11 @@
 
 #define SILENT_WINDOW_OPEN_LIMIT 5
 
+#include <gnome.h>
+
+#include <string.h>
 #include <signal.h>
+#include <sys/wait.h>
 
 #include "gsearchtool.h"
 #include "gsearchtool-support.h"
@@ -151,11 +155,19 @@ click_stop_cb (GtkWidget 	*widget,
 	gtk_widget_set_sensitive (interface.find_button, TRUE);
 }
 
+void
+size_allocate_cb (GtkWidget	*widget,
+		  GtkAllocation *allocation,
+	      	  gpointer 	data)
+{
+	gtk_widget_set_size_request (interface.add_button,
+			      	     allocation->width,
+			             allocation->height);
+}
 void
 add_constraint_cb (GtkWidget 	*widget, 
 		   gpointer 	data)
 {
-	SearchConstraint *constraint = g_new(SearchConstraint,1);
 	GtkWidget *menu;
 	GtkWidget *item;
 	
@@ -480,7 +492,6 @@ file_button_release_event_cb (GtkWidget 	*widget,
 		      	      gpointer 		data)
 {
 	gboolean no_files_found = FALSE;	
-	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 
 	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW(interface.tree))) {
@@ -555,7 +566,6 @@ file_event_after_cb  (GtkWidget 	*widget,
 		      GdkEventButton 	*event, 
 		      gpointer 		data)
 {	
-	GtkTreeSelection *selection;
 	GtkTreeIter 	 iter;
 
 	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW(interface.tree))) {
@@ -755,26 +765,50 @@ save_results_cb (GtkFileSelection *selector,
 	
 	interface.save_results_file = (gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(interface.file_selector));
 	
-	if (access (interface.save_results_file, F_OK) == 0) {
-		GtkWidget *dialog;
-		gint response;
+	if (g_file_test (interface.save_results_file, G_FILE_TEST_EXISTS) == TRUE) {
 		
-		dialog = gtk_message_dialog_new
-			(GTK_WINDOW (interface.file_selector),
-			 0 /* flags */,
-			 GTK_MESSAGE_QUESTION,
-			 GTK_BUTTONS_YES_NO,
-			 _("File \"%s\" already exists. Overwrite?"),
-			 interface.save_results_file);
-		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW (interface.main_window));
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		if (g_file_test (interface.save_results_file, G_FILE_TEST_IS_DIR) == TRUE) {
+			GtkWidget *dialog;
+			gchar *error_msg = g_strdup_printf (_("Cannot save search results.\n\"%s\" is a folder."),
+							      interface.save_results_file);
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_MESSAGE_ERROR,
+					GTK_BUTTONS_OK,
+					error_msg);
+
+			g_signal_connect (G_OBJECT (dialog),
+					"response",
+					G_CALLBACK (gtk_widget_destroy), NULL);
+					
+			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+			gtk_widget_show (dialog);
+			g_free (error_msg);
+			
+			return;	
+		}
+		else {
+			GtkWidget *dialog;
+			gint response;
 		
-		gtk_widget_destroy (GTK_WIDGET(dialog));
+			dialog = gtk_message_dialog_new
+				(GTK_WINDOW (interface.file_selector),
+				 0 /* flags */,
+				 GTK_MESSAGE_QUESTION,
+				 GTK_BUTTONS_YES_NO,
+				 _("File \"%s\" already exists. Overwrite?"),
+				 interface.save_results_file);
+			gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW (interface.main_window));
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
 		
-		if (response != GTK_RESPONSE_YES) return;
+			gtk_widget_destroy (GTK_WIDGET(dialog));
+		
+			if (response != GTK_RESPONSE_YES) return;
+		}
 	}
 	
-	if (fp = fopen(interface.save_results_file, "w")) {
+	if ((fp = fopen (interface.save_results_file, "w")) != NULL) {
 	
 		if (gtk_tree_model_get_iter_root(GTK_TREE_MODEL(store), &iter)) {
 			
@@ -840,7 +874,7 @@ key_press_cb (GtkWidget    	*widget,
 	      GdkEventKey	*event, 
 	      gpointer 		data)
 {
-	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
 
 	if (event->keyval == GDK_Escape) {
 		if (search_command.running == RUNNING) {
@@ -852,20 +886,24 @@ key_press_cb (GtkWidget    	*widget,
 	}
 	else if (event->keyval == GDK_F10) {
 		if (event->state & GDK_SHIFT_MASK) {
-			GtkWidget *popup;
-			gchar *utf8_name = NULL;
-			gchar *utf8_path = NULL;
 			gboolean no_files_found = FALSE;
+			GtkWidget *popup;
 			GtkTreeIter iter;
+			GList *list;
+			
+			if (gtk_tree_selection_count_selected_rows (GTK_TREE_SELECTION(interface.selection)) == 0) {
+				return FALSE;
+			}
 		
-			if (!gtk_tree_selection_get_selected (interface.selection, NULL, &iter))
-    				return FALSE;
+			list = gtk_tree_selection_get_selected_rows (GTK_TREE_SELECTION(interface.selection),
+						     (GtkTreeModel **)&interface.model);
+			
+			gtk_tree_model_get_iter (GTK_TREE_MODEL(interface.model), &iter, 
+						 g_list_first(list)->data);
 		
 			gtk_tree_model_get (GTK_TREE_MODEL(interface.model), &iter,
-					    COLUMN_NAME, &utf8_name,
-					    COLUMN_PATH, &utf8_path,
-				    	COLUMN_NO_FILES_FOUND, &no_files_found,
-				    	-1);
+					    COLUMN_NO_FILES_FOUND, &no_files_found,
+				    	    -1);
 				    
 			if (!no_files_found) {
 				popup = gnome_popup_menu_new (popup_menu);
@@ -883,7 +921,7 @@ look_in_folder_key_press_cb (GtkWidget    	*widget,
 		     	     GdkEventKey	*event, 
 		     	     gpointer 		data)
 {
-	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
 
 	if (event->keyval == GDK_Return)
 	{
